pub trait PacketGenerator {
// Required method
fn packets(&self) -> Vec<Vec<u8>>;
}Expand description
A trait for packet generation that can be used across different protocols
This trait defines the core functionality for generating packet bytes that will be transmitted to Timex Datalink devices.